# Gli oggetti di una sessione di lavoro R rimangono in memoria (i.e. nel workspace).
# Prima di iniziare una nuova sessione di analisi, e' opportuno "svuotare"
# lo spazio di lavoro
# Riferimenti: ?rm
rm(list=ls())

# Risulta comodo immagazzinare dati, script e risultati nella stessa directory.
#  Conviene utilizzare una directory separata per ciascuna analisi.
#  Spesso - pero' - succede di dover spostare i vari file, e al contempo non 
# desiderabile dover rivedere l'intero script a causa del cambiamento del
# nome di un file o dello spostamento di dati e script in una differente
# directory.
#  Una tecnica elementare consiste nel dichiarare in anticipo gli 'elementi
# variabili', in pratica definendoli come costanti. In caso di variazioni
# sara' sufficiente sistemare i nuovi valori in un punto solo dello script.
#   Definiamo come costante la directory nella quale stanno i dati
##@TODO modificare rispetto alla propria directory di lavoro
workdir <- "/data/Didattica/R/R_2013/src/"
# definisce la 'cartella di lavoro' di R
setwd(workdir)
##@Q&A: e se non uso setwd()? Dove 'lavora' (ad esempio salva i risultati) R?


##@TODO modificare togliendo in carattere di commento a uno (e uno solo!)
##@TODO dei due blocchi di codice seguenti

## legge i dati direttamente da un foglio Excel (occorre avere installato 
## il package XLConnect
#require(XLConnect)
#data <- readWorksheetFromFile(paste("data","hares.xls", sep="/"), sheet="totale")

## legge i dati da un file di testo CSV (Comma Separated Values, occorre avere
## il package foreign)
library(foreign)
data <- read.csv(paste("data","hares.csv", sep="/"))

##@Q&A: che differenza c'e' tra i comandi require() e library()? 

##@Q&A: cha cosa fa il comando paste()? Provare a copiare e incollare nella linea di comando solo la funzione paste("data","SOVRAPP_LC-VAR.def.csv", sep="/")

##@Q&A: che razza di oggetto e' data, dal momento che non  stato specificato nulla di particolare?
#Riferimenti: ?data.frame

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# rapido esame della struttura dei dati
head(data)
str(data)

##@Q&A: quante osservazioni ho per ciascuna specie?

##@Q&A: quante osservazioni ho per anno?

##@Q&A: e combinando specie e anno?
#Riferimenti: ?xtabs, ?formula

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Condizionamento dei dati
##@Q&A: le variabili sono del 'tipo' corretto?
#Specie
#Comprensorio
#Quota
#Anno
#Riferimenti: ?class, ?is, ?str

# Sarebbe meglio se Specie e Comprensorio fossero variabili di tipo 'factor' anzich di tipo carattere

##@Q&A: E' possibile fare di meglio, rispetto a usare delle sigle incomprensibili?
#  Una variabile di tipo factor possiede dei _livelli_, codificati con numeri interi
# e a ciascun livello pu essere associata un'_etichetta_ di comodo, che viene
# utilizzata ad esempio nei titoli delle tabelle e nei grafici
#Riferimenti: ?factor
data$Specie <- factor(data$Specie, levels=c('lepcom','lepvar'), labels=c('L. europaeus','L. timidus'))

data$Comprensorio <- factor(data$Comprensorio)

#  La variabile Comprensorio ha un numero di livelli non noto. Sarebbe meglio
# esaminare i valori esistenti e quindi ricodificarli con qualcosa di piu' comprensibile
#  I comandi utilizzati sono presentati esclusivamente a titolo di esempio.
# In questo caso, e' comune in uno script R prima del blocco di codice commentato che
# funge da esempio, ma non e' essenziale ai fini della procedura di elaborazione,
# aggiungere un commento che tradizionalmente indica che il codice seguente e'
# appunto un esempio e non  importante o necessario eseguirlo.

## Not Run:
# summary(data$Comprensorio)
#> AV  CH  MO  SO  TI
#>157  94 218 314 394

data$Comprensorio <- factor(data$Comprensorio, levels=c("CH", "MO", "SO", "TI", "AV"), labels=c("Val Chiavenna", "Morbegno", "Sondrio", "Tirano", "Alta Valle"))

## Not Run:
# summary(data$Comprensorio)

##@Q&A Il 'cambio di etichette' dovrebbe dare un risultato migliore ad esempio
##@Q&A nel caso delle tabulazioni...

## Not Run:
# xtabs( ~ Specie + Comprensorio, data)

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Analisi esplorativa: le quote medie almeno 'sembrano' diverse o no?
# si potrebbe pensare che sia necessario 'aggregare' le osservazioni in modo 
# da avere un 'totale' per ciascuna quota. A pensarci bene, e' fondamentalmente
# tempo perso: e' molto piu' pratico lavorare sui dati grezzi!

# Eccoun esempio di inutile aggregazione
## Not Run:
## (data.freq <- data.frame(xtabs(~Quota+Specie, data)))

# il sistema piu' rapido per visualizzare 'le medie' di piu' 'gruppi' e' il box-and-whisker plot
boxplot(Quota ~ Specie,data)

# Il package lattice e' in grado di produrre grafici piu' complessi e in modo piu' comodo.
#   Lattice usa un'interfaccia a formula!
#   In lattice, il comando equivalente a boxplot e' bwplot
bwplot(Quota ~ Specie,data)

##@Q&A,@TODO perche' il comando bwlpot non funziona? E' presente ed e' stato caricato il package lattice?

# ggplot2 e' ancora meglio, da un punto di vista estetico
#   GGplot2 si basa sulla metodologia creata da Leland Wilkinson nota come "grammar of Graphics"
ggplot(data, aes(x=Specie, y=Quota)) + geom_boxplot()

##@Q&A,@TODO E' stata caricata la libreria ggplot2?

# Un altro modo di visualizzare i dati, utilizzando la grafica a pannelli di lattice
histogram(~Quota | Specie, data=data, type="count", layout=c(1,2))
histogram(~Quota | Comprensorio + Specie, data=data, type="count")
##@TODO provare ad invertire l'ordine dei fattori nel comando precedente
histogram(~Quota | Anno + Specie, data=data, type="count")

# in ggplot2
ggplot(data, aes(x=Quota)) + geom_histogram() + facet_grid(Specie ~ .)
ggplot(data, aes(x=Quota)) + geom_histogram() + facet_grid(Specie ~ Comprensorio)

# Un altro modo di visualizzare i dati, utilizzando uno specifico grafico
# ('violin plot')
require(vioplot)
vioplot(data[data$Specie == "L. europaeus","Quota"], ##@Q&A: che significano quei nomi di variabile racchiusi tra quadre?
        data[data$Specie == "L. timidus","Quota"],
        names= c('L. europaeus','L. timidus'),       # specifica i nomi degli assi
        col='lightgray',                             # (quasi) nessuna rivista pubblica grafici a colori!
        ylim=c(0,3000)                               # forziamo la scala dell'asse Y su un intervallo 'comodo'
       )


#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Per avere un'evidenza statistica delle differenze, occorre eseguire un test 
# sulla differenza tra le medie.
#  Ad esempio, il t di Student, che pero' richiede che le variabili oggetto del
# test abbiano distribuzione normale.
#  Prima ancora - quindi - occorrer effettuare un test per la normalita' (ad 
# esempio il test di Shapiro-Wilks).
#Riferimenti: ?shapiro.test
shapiro.test(data[data$Specie == "L. europaeus","Quota"])
shapiro.test(data[data$Specie == "L. timidus","Quota"])
##@Q&A: sono distribuite normalmente? Indizio: cosa e' possibile capire dall'
##@Q&A: esempio nella pagina di manuale di shapiro.test?

# Un'alternativa (che non e' un test statistico) in forma grafica: il quantile-
# quantile plot mette in grafico i valori di una data variabile rispetto ai 
# valori che quella variabile avrebbe _se fosse_ normale.
#  Riferimenti: ?qqnorm, ?qqplot, ?qline
qqnorm(data[data$Specie == "L. europaeus","Quota"])
qqline(data[data$Specie == "L. europaeus","Quota"],col='red')
qqnorm(data[data$Specie == "L. timidus","Quota"])
qqline(data[data$Specie == "L. timidus","Quota"],col='red')

##@Q&A: c'e' per caso qualche ripetizione?

# Creare del codice contenente ripetizioni e' fondamentalmente una tecnica suicida,
# in particolare nel caso in cui si debbano fare in seguito delle modifiche.
#  La soluzione intelligente e' quella di creare una _funzione_, per poi utilizzarla
# tutte le volte che si vuole.

myqqplot <- function(x) {
  qqnorm(x)
  qqline(x,col='red')
}

myqqplot(data[data$Specie == "L. europaeus","Quota"])
myqqplot(data[data$Specie == "L. timidus","Quota"])

# c'e' ancora qualche ripetizione...
lapply(split(data$Quota, data$Specie) , myqqplot)

##@Q&A: E' possibile verificare la differenza tra quote con un t-test?
# Riferimenti: ?t.test

## Not Run:
##t.test(Quota ~ Specie,data)
