# 'pulizia' workspace
rm(list=ls())
##@TODO modificare rispetto alla propria directory di lavoro
workdir <- "/data/Didattica/R/R_2013/src/"
setwd(workdir)

# caricamento dati
data <- read.table('data/terrapin_vs_slider.txt', header=TRUE)


################################################################################
## Nota
## I comandi utilizzati per le varie fasi dell'analisi sono il più delle volte
## commentati (Not Run:). Confrontare ciascun comando con le rispettive pagine
## di manuale, e, una volta compreso il funzionamento del comando, togliere i
## caratteri di commento ed eseguire il comando.
## È anche possibile trovare delle soluzioni alternative...


##Q&A: Le misure di temperatura sono state effettuate ogni 15'.
##Q&A: Che cosa c'è che non va nella variabile data$tempo e com'è possibile sistemarlo?
# Soluzione: tempo è stato codificato come numero sequenziale, che si riferisce
# a intervalli successivi di 15'. È sufficiente moltiplicare per 15!

timespan <- 15 # minutes
data$tempo <- data$tempo * timespan


##Q&A: Come varia la temperatura in funzione del tempo?
##Q&A: È possibile visualizzare le curve di temperatura separatemante per le due specie?
# Soluzione: quando si tratta di 'ripetere lo stesso grafico' in funzione di una
# variabile fattoriale (in questo caso data$specie), è il caso di usare lattice.
require(lattice)
xyplot(temperatura~tempo, group=specie, type=c('r','p'), data, auto.key=TRUE)
bwplot(temperatura~tempo| specie, data, horizontal=FALSE)

# Soluzione (2): utilizzare un conditioning plot, facendo passare una polinomiale
#  per i punti sperimentali
# Riferimenti: ?coplot, ?panel.smooth
## Not Run:
## coplot(temperatura ~ tempo | specie, data, panel=panel.smooth)
## coplot(temperatura ~ tempo | specie * sesso, data, panel=panel.smooth)

# Soluzione (3): panel plot in ggplot2, facendo passare una polinomiale
#  per i punti sperimentali
# Riferimenti: ?ggplot, ?stat_smooth
require(ggplot2)
require(scales)
plt <- ggplot(data, aes(x=tempo, y=temperatura, color=specie)) + geom_point()
plt + facet_grid(specie ~ .)
plt + facet_grid(. ~ specie)
plt + facet_grid(specie ~ sesso)
plt + facet_grid(specie ~ sesso) + stat_smooth(method='lm')
plt + facet_grid(specie ~ sesso) + stat_smooth(method='loess')

##Q&A: Qual è il modello più adatto per rappresentare la variazione di temperatura?
##Q&A: (In altri termini: è sufficiente una retta?)
# Riferimenti: ?lm (regressione lineare), ?nls (regressione non lineare)

##Q&A: Le analisi vanno ripetute _due_ volta, una per ciascuna specie.
##Q&A:  È possibile identificare una soluzione al problema che utilizzi delle
##Q&A: funzioni?


################################################################################
## Soluzione 1 - regressione lineare
# non viene utilizzata una funzione, ma un approccio _iterativo_ mediante una 
# iterazione finita (comando 'for').
# Riferimenti: ?Control

# Il comando for deve avere una variabile _iterabile_ (un vettore, una lista, ecc.)
# in base alla quale ripetere di volta in volta.
# Creiamo un elenco delle specie.
# Riferimenti: ?unique, ?as.character
lista.specie <- as.character(unique(data$specie))

# Se eseguiamo le analisi in modo iterativo, non è possibile accedere
# ai risultati se non quando il ciclo for è terminato.
#  Occorre quindi creare un 'contenitore' che riceva i dati mentre il ciclo è in
# funzione, in modo da poterli consultare in seguito.
#  Creiamo una lista, vuota (per ora!)
# Riferimenti: ?list
results <- list()

# ciclo for sugli elementi di lista.specie per il calcolo della regressione lineare
for (s in lista.specie) {                    # ripetere per ciascuna specie
  d <- subset(data,subset=specie==s)         # estrae i dati relativi alla specie 's' in un dataframe
  results[[s]] <- lm(temperatura ~ tempo, d) # esegue la regressione lineare, e immagazzina il risultato
}

# un pò meglio, usando lapply, senza 'for'
splist <- split(data, data$specie)
results2 <- lapply(splist, function(x) lm(x$temperatura ~ x$tempo))

##Q&A: Che cosa contengono adesso gli oggetti results e results2?
# Riferimenti: ?str, ?names
# Nota: per accedere a un elemento di una lista, si utilizza la notazione con
#  le doppie parentesi quadre (vedi riga 89 sopra)

##Q&A: Come si fa a vedere i risultati delle regressioni?
# Riferimenti: ?summary, ?plot

## Not Run:
## visualizza i risultati delle regressioni
# summary(results[['trachemys']])
# summary(results[['emys']])

## Not Run:
## visualizza i risultati delle regressioni
# plot(results[['trachemys']])
# plot(results[['emys']])

################################################################################
## Soluzione 2 - regressione non lineare
# Modello: esponenziale negativa
#  In questo caso, all'interno del ciclo sono stati inseriti comandi per la 
# visualizzazione immediata dei risultati.
results.nls <- list()
for (s in lista.specie) {
  d <- subset(data,subset=specie==s)
  # regressione non lineare (notare i 'guess values' per i parametri del modello)
  results.nls[[s]] <- nls(temperatura ~ Tmin + Tdiff * (1 - exp(-(k/100)*tempo)), d, start=list(Tmin=19,Tdiff=5,k=1))
  cat("================================================================================\n","Non linear regression for id = ", s, "\n\n")
  print(summary(results.nls[[s]]))
  cat("\n\n")
}

## stima dei parametri: viene eseguita una regressione non lineare per ciascun animale
## i coefficienti e i valori dei parametri vengono immagazzinati in un dataframe
## creato in precedenza

# struttura per immagazzinare i risultati
animali <- unique(data[,c('id','specie', 'sesso')])
animali$AIC <- NA     # Akaike Information Criterion
animali$Tmin <- NA  # parametro: T iniziale
animali$Tmax <- NA  # parametro: delta T
animali$k <- NA     # parametro: tasso di riscaldamento

# calcolo iterativo delle regressioni
for (b in animali$id) {
  d <- subset(data,subset=id==b)
  r <- nls(temperatura ~ Tmin + Tdiff * (1 - exp(-(k/100)*tempo)), d, start=list(Tmin=19,Tdiff=5,k=1))
  cat("================================================================================\n","Non linear regression for id = ", b, "\n\n")
  print(summary(r))
  cat("\n\n")
  animali[animali$id == b,]$AIC <- AIC(r)
  animali[animali$id == b,]$Tmin <- coef(r)[['Tmin']]
  animali[animali$id == b,]$Tmax <- coef(r)[['Tdiff']] + coef(r)[['Tmin']]
  animali[animali$id == b,]$k <- coef(r)[['k']]
}

# analisi della varianza: k è differente tra specie?
cat("================================================================================\n","ANOVA for k (negative exponential model)\n\n")
aov.k <- aov(k ~ specie * sesso, animali, family='gaussian')
summary(aov.k)
TukeyHSD(aov.k)
bwplot(k ~ specie , animali)

# analisi della varianza: Tmax è differente tra specie?
cat("================================================================================\n","ANOVA for k (negative exponential model)\n\n")
aov.Tmax <- aov(Tmax ~ specie * sesso, animali)
summary(aov.Tmax)
TukeyHSD(aov.Tmax)
bwplot(Tmax ~ sesso | specie, animali)
## End of File ##