###########################################################
### Testing conifer cones value acrosss areas and years ###
###########################################################
#Q: cones production of different conifers species has been registered in several sites and in consecutive years. Cones production change across years... why? 
#1. Seed predator or 2.Climate change? If climate change influence cones production is there any correlation between species, years and different areas? 
# 'clean' workspace
rm(list=ls())

########WORKING ENVIRONMENT SETTINGS###########
##@TODO change according to your workdir 
workdir <- "/data/Didattica/R/R_2014/src/data"
setwd(workdir)

##ATTENZIONE: i pacchetti che permettono di leggere e gestire i file xls/xlsx si possono appoggiare ad altri programmi che devono essere presenti sulla vostra macchina. la funziona read.xls del pacchetto gdata utilizza il programma perl. Se utilizzate SO Windows dovete specificare al comando dove risiede Perl. Se il vostro SO ? UNIX non ? necessario 
#perl="C:/Perl64/bin/perl.exe"##it is necessary to read xls files, at least on Windows...


#######PACKAGES#########
library(gdata)##read xls
library(ggplot2)## graphics package
library(reshape2)## modify dataset
#library(XLConnect)## read and write xls and xlsx (it needs java)

#########################################################
#######LOADING DATA######################################
#########################################################

#####CONE PER PLOT#####################################
xlsxfile <- 'Pigne.xlsx'
#vegx <- read.xls(xlsxfile, sheet=1, stringsAsFactors=FALSE, perl=perl)## su windows
vegx <- read.xls(paste(workdir,'data',xlsxfile, sep='/'), sheet=1, stringsAsFactors=FALSE)

### have a look of data
str(vegx)
head(vegx)

########## FIXING DATA FOR ANALYSIS ################
vegx[vegx$tree.species%in% 'P.abies',]$tree.species <- 'P. abies'
vegx[vegx$Area%in% 'CEDRASCO',]$Area <- 'CED'
vegx$cones <- as.integer(vegx$cones)#number of cones is a character variable, but we need it as numeric
vegx <- vegx[!is.na(vegx$cones),]#dropping cones NaN values, 
##ATTENTION: be careful at the differences between 0 (zero) and NA!

#########################################################
####folding data to create proper dataframe##############
#########################################################

## we need to have mean, standard deviation and CV of number of cones 
vegmean <- aggregate(cones~Area+year+tree.species,vegx,FUN=mean)
names(vegmean) <- c("Area","year","tree.species","mean")
vegsd <- aggregate(cones~Area+year+tree.species,vegx,FUN=sd)
names(vegsd) <- c("Area","year","tree.species","sd")
vegtot <- cbind(vegmean, vegsd$sd)
names(vegtot) <- c("Area","year","tree.species","mean",'sd')
vegtot$cv <- vegtot[, "mean"] / vegtot[, "sd"]

############################################
##can we optimize the code above?###########
############## TRY #########################
#vegtot <- setNames(aggregate(cones~Area+year+tree.species,vegx,FUN= function(x) c(mean(x), sd(x))), c("Area","year","tree.species","mean",'sd'))
#names(vegtot) <- c("Area","year","tree.species","mean",'sd')

## as usual while your are running your experiments you get new data and most of the time format is different...
####################################
#### load other data  ##############
####################################
xlsvda <- 'Pigne2.xls'
###read in from the same file two different sheets ###
#rhemes <- read.xls(paste(workdir,'data',xlsvda, sep='/'), sheet='Rhemes', stringsAsFactors=FALSE, perl=perl) ##windows
rhemes <- read.xls(paste(workdir,'data',xlsvda, sep='/'), sheet='Rhemes', stringsAsFactors=FALSE)
#cogne <- read.xls(paste(workdir,'data',xlsvda, sep='/'), sheet='Cogne', stringsAsFactors=FALSE, perl=perl) ##windows
cogne <- read.xls(paste(workdir,'data',xlsvda, sep='/'), sheet='Cogne', stringsAsFactors=FALSE)

##have a look of the new dataset...
str(rhemes)
str(cogne)

##obviously format is different...
##FIX the new dataset according to previous dataframe
vdarh <- melt(rhemes, id=c("Area", "Area.code","ID",'Trap','Tree.species'),variable.name = "year")## metl dataset according to previous dataframe
vdarh$year <- substring(as.character(vdarh$year),2,5)##something strange with year!

vdaco <- melt(cogne, id=c("Area", "Area.code","ID",'Trap','Tree.species'),variable.name = "year")## melt dataset according to previous dataframe
vdaco$year <- substring(as.character(vdaco$year),2,5)##something strange with year!

vda <- rbind(vdarh,vdaco)##join of the two dataframe
names(vda) <- c('Area', "Area.code","ID",'Trap','tree.species', 'year','cones')
vdamean <- aggregate(cones~Area+year+tree.species,vda,FUN=mean)
names(vdamean) <- c("Area","year","tree.species","mean")
vdasd <- aggregate(cones~Area+year+tree.species,vda,FUN=sd)
names(vdasd) <- c("Area","year","tree.species","sd")
vdatot <- cbind(vdamean, vdasd$sd)
names(vdatot) <- c("Area","year","tree.species","mean",'sd')
vdatot$cv <- vdatot[, "mean"] / vdatot[, "sd"]

###what if factor levels are different??###
vdatot[vdatot$tree.species%in%'abete',]$tree.species <- 'P. abies'
vdatot[vdatot$tree.species%in%'Abete',]$tree.species <- 'P. abies'
vdatot[vdatot$tree.species%in%'larice',]$tree.species <- 'Larix'
vdatot[vdatot$tree.species%in%'Larice',]$tree.species <- 'Larix'
vdatot$year <- as.factor(vdatot$year)

###Finally the complete dataframe!###
vegetation <- rbind(vegtot,vdatot)

## Q: Is any correlation between cones production and area+species####
##prepare data for correlation analysis
vegetation$tree.species <- as.factor(vegetation$tree.species)

##dcast stretch dataframe
zz <- dcast(vegetation, year ~ Area+tree.species, value.var='mean',fun.aggregate=mean)
##have a look of the matrix
str(zz)
head(zz)
##correlation test
tt <- cor(zz[,2:19], use="pairwise.complete.obs")

##################################
###write matrix to file###########
##################################
##package XLConnect it is useful not just to read but also to write matrix
##where does it write it?
writeWorksheetToFile('cor_cones_speciesXarea.xls', tt, 'spec_vs_area', styleAction = XLC$STYLE_ACTION.XLCONNECT)

writeWorksheetToFile('matrice_pigne.xls', zz, 'spec_x_area', styleAction = XLC$STYLE_ACTION.XLCONNECT)

############################
####some useful graphics####
############################
vegetation$year <- as.factor(vegetation$year)

##base plot
plot.0 <- ggplot(vegetation,aes(x=year,y=mean,fill=tree.species))+facet_wrap(~tree.species,scale='free_y')+geom_bar(stat = "identity")
plot.0

##Fix axes text and rearrange scale
plot.1 <- plot.0 +theme(axis.text.x = element_text(angle = 90, hjust = 0.5))
plot.1

##Fix axes labes
plot.2 <- plot.1 + ylab('Number of Cones')+ xlab('Year')
plot.2

##Remove legend
plot.3 <- plot.2 + theme(legend.position="none")
plot.3

##changing plot placement
plot.4 <- plot.3 + facet_wrap(~tree.species,scale='free_y',ncol = 1)

##underline mast year
plot.5 <- plot.4 + geom_vline(xintercept=as.numeric(vegetation$year[c(1,3,6,8,45)]),linetype = "longdash")
plot.5

##add title and print out graphic choosing dimension and resolution
plot <- plot.5 +ggtitle('MAST')
png(filename='PLOT.png', width=15, height=20, units='cm', res=300)
plot
dev.off()

